@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,9 +27,12 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <gsl/gsl_version.h>
 #include "termcol.h"
 #include "Error.h"
 
+extern void
+ilaver_(int *vers_major, int *vers_minor, int *vers_patch);
 
 static void
 Expert1(void)
@@ -106,6 +109,10 @@ Expert2(void)
 void
 Usage(int expert)
 {
+//    int vers_major, vers_minor, vers_patch;
+    
+//    ilaver_(&vers_major, &vers_minor, &vers_patch);
+
     PrintTheseusPre();
     printf("  Usage:                                                                        \n");
     printf("    theseus [options] <pdb files>                                               \n\n");
@@ -162,9 +169,11 @@ Usage(int expert)
     printf("\n");
     printf("   Douglas L. Theobald and Deborah S. Wuttke (2008) \n");
     printf("   \"Accurate structural correlations from maximum likelihood superpositions.\"\n");
-    printf("   PLOS Computational Biology, in press\n");
+    printf("   PLOS Computational Biology, 4(2):e43\n");
     printf("\n");
     printf("   http://www.theseus3d.org/\n");
+    printf("   Compiled with GSL version %s\n", GSL_VERSION);
+    //printf("   Compiled with LAPACK version %d.%d.%d\n", vers_major, vers_minor, vers_patch);
     printf("I===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-==I\n");
     printf("                            %s<  END THESEUS %s  >%s \n\n\n",
            tc_RED, VERSION, tc_NC);
@@ -199,5 +208,6 @@ Version(void)
 {
     printf("\n  THESEUS version %s compiled on %s %s\n  by user %s with machine \"%s\" \n\n",
            VERSION, __DATE__, __TIME__, getenv("USER"), getenv("HOST"));
+    printf("  Compiled with GSL version %s\n", GSL_VERSION);
     fflush(NULL);
 }
