@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -74,15 +74,18 @@ FragCoords
 
 
 void
-FragCoordsFree(FragCoords *frag)
+FragCoordsFree(FragCoords **frag_ptr)
 {
+    FragCoords *frag = *frag_ptr;
+
     free(frag->x);
     free(frag->y);
     free(frag->z);
     free(frag->var);
     free(frag->w);
     free(frag);
-    frag = NULL;
+
+    *frag_ptr = NULL;
 }
 
 
