@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,6 +34,7 @@
 #include "QuarticHornFrag.h"
 #include "FragDist.h"
 
+
 void
 FragDist(CoordsArray *cdsA, int fraglen, int center_ca)
 {
@@ -55,7 +56,7 @@ FragDist(CoordsArray *cdsA, int fraglen, int center_ca)
 
     offset = (fraglen - 1) / 2;
 
-    num = cdsA->vlen * (mysquare(cdsA->cnum) - cdsA->cnum) / 2;
+    num = cdsA->vlen * cdsA->cnum * (cdsA->cnum - 1) / 2;
 
     array = (double *) calloc(num,  sizeof(double));
 
@@ -105,8 +106,8 @@ FragDist(CoordsArray *cdsA, int fraglen, int center_ca)
         }
     }
 
-    FragCoordsFree(frag1);
-    FragCoordsFree(frag2);
+    FragCoordsFree(&frag1);
+    FragCoordsFree(&frag2);
     free(coeff);
     fclose(distfile);
     fclose(distfile2);
