@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -290,7 +290,7 @@ GetSeq2PDBMap_old(PDBCoordsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 static void
 GetSeq2PDBMap(PDBCoordsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 {
-    int             i, j, k;
+    int             i, j, k, str1len, str2len;
     const int       cnum = pdbA->cnum;
     char            filename_root[FILENAME_MAX], msaname_root[FILENAME_MAX], seqname_root[FILENAME_MAX];
 
@@ -307,13 +307,19 @@ GetSeq2PDBMap(PDBCoordsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
             strncpy(filename_root, getroot(pdbA->coords[i]->filename), strlen(pdbA->coords[i]->filename)+1);
             strncpy(seqname_root, getroot(seq2pdb->pdbfile_name[j]), strlen(seq2pdb->pdbfile_name[j])+1);
 
-            if (strncmp(filename_root, seqname_root, strlen(seqname_root)) == 0)
+            str1len = strlen(seqname_root);
+            str2len = strlen(filename_root);
+            if ((strncmp(filename_root, seqname_root, strlen(seqname_root)) == 0) &&
+                (str1len == str2len))
             {
                 for (k = 0; k < msa->seqnum; ++k)
                 {
                     strncpy(msaname_root, getroot(msa->name[k]), strlen(msa->name[k])+1);
 
-                    if (strncmp(seqname_root, msaname_root, strlen(msaname_root)) == 0) /* now k (alignment) and i (pdb coords) match */
+                    str1len = strlen(msaname_root);
+                    str2len = strlen(seqname_root);
+                    if ((strncmp(seqname_root, msaname_root, strlen(msaname_root)) == 0) &&
+                        (str1len == str2len)) /* now k (alignment) and i (pdb coords) match */
                     {
                         pdbA->seq2pdb->map[i] = k;
                         msa->flag[k] = 0; /* reset msa flag to 0 since this structure exists */
@@ -876,7 +882,8 @@ pdb2fst(PDBCoordsArray *pdbA)
 
         fp = fopen(outfile, "w");
 
-        fprintf(fp, ">%-72s", pdbA->coords[i]->filename);
+        //fprintf(fp, ">%-72s", pdbA->coords[i]->filename);
+        fprintf(fp, ">%-s", pdbA->coords[i]->filename);
 
         count = 0;
         for (j = 0; j < pdbA->coords[i]->vlen; ++j)
