@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "pdbStats.h"
 #include "CovMat.h"
 #include "DLTmath.h"
+#include "FragCoords.h"
 #include "ProcLAPACKSVD.h"
 
 #if defined(__APPLE__)
@@ -718,6 +719,112 @@ ProcLAPACKSVDvan(const Coords *coords1, const Coords *coords2, double **rotmat,
 }
 
 
+static void
+CalcRFrag(const FragCoords *coords1, const FragCoords *coords2, double **Rmat)
+{
+    int             i;
+    const double   *x1 = (const double *) coords1->x,
+                   *y1 = (const double *) coords1->y,
+                   *z1 = (const double *) coords1->z;
+    const double   *x2 = (const double *) coords2->x,
+                   *y2 = (const double *) coords2->y,
+                   *z2 = (const double *) coords2->z;
+    double          x2i, y2i, z2i, x1i, y1i, z1i;
+    double          Rmat00, Rmat01, Rmat02,
+                    Rmat10, Rmat11, Rmat12,
+                    Rmat20, Rmat21, Rmat22;
+
+   /*  printf("\n axes weights: %f %f %f ", axeswx, axeswy, axeswz); */
+
+    Rmat00 = Rmat01 = Rmat02 = Rmat10 = Rmat11 = Rmat12 =
+    Rmat20 = Rmat21 = Rmat22 = 0.0;
+
+    i = coords1->fraglen;
+    while(i-- > 0)
+    {
+        x1i = *x1++;
+        y1i = *y1++;
+        z1i = *z1++;
+
+        x2i = *x2++;
+        y2i = *y2++;
+        z2i = *z2++;
+
+        Rmat00 += x2i * x1i;
+        Rmat01 += x2i * y1i;
+        Rmat02 += x2i * z1i;
+        
+        Rmat10 += y2i * x1i;
+        Rmat11 += y2i * y1i;
+        Rmat12 += y2i * z1i;
+        
+        Rmat20 += z2i * x1i;
+        Rmat21 += z2i * y1i;
+        Rmat22 += z2i * z1i;
+    }
+
+    Rmat[0][0] = Rmat00;
+    Rmat[0][1] = Rmat01;
+    Rmat[0][2] = Rmat02;
+    Rmat[1][0] = Rmat10;
+    Rmat[1][1] = Rmat11;
+    Rmat[1][2] = Rmat12;
+    Rmat[2][0] = Rmat20;
+    Rmat[2][1] = Rmat21;
+    Rmat[2][2] = Rmat22;
+}
+
+
+static double
+CalcInnProdNormFrag(const FragCoords *coords)
+{   
+    int             i;
+    double          sum;
+    const double   *x = (const double *) coords->x,
+                   *y = (const double *) coords->y,
+                   *z = (const double *) coords->z;
+    double          xi, yi, zi;
+
+    sum = 0.0;
+    i = coords->fraglen;
+    while(i-- > 0)
+    {
+        xi = *x++;
+        yi = *y++;
+        zi = *z++;
+
+        sum += xi * xi + yi * yi + zi * zi;
+    }
+
+    return(sum);
+}
+
+
+/* returns sum of squared residuals, E
+   rmsd = sqrt(E/atom_num)  */
+double
+ProcLAPACKSVDFrag(const FragCoords *frag1, const FragCoords *frag2, double **rotmat,
+                  double **Rmat, double **Umat, double **VTmat, double *sigma)
+{
+    double          det, sumdev, term1, term2;
+
+    term1 = CalcInnProdNormFrag(frag2);
+    CalcRFrag(frag1, frag2, Rmat);
+    CalcLAPACKSVD(Rmat, Umat, sigma, VTmat);
+    det = CalcRotMat(rotmat, Umat, VTmat);
+    term2 = CalcInnProdNormFrag(frag1);
+    sumdev = term1 + term2;
+
+    if (det < 0)
+        sumdev -= 2.0 * (sigma[0] + sigma[1] - sigma[2]);
+    else
+        sumdev -= 2.0 * (sigma[0] + sigma[1] + sigma[2]);
+
+    return(sumdev);
+}
+
+
+
 double
 ProcLAPACKSVD(const Coords *coords1, const Coords *coords2, double **rotmat,
               const double *weights, const double *axesw,
