@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #define QUARTICHORN_SEEN
 
 #include "FragCoords.h"
+#include "Coords.h"
 
 void
 ComputeQuartHornFrag(const FragCoords *coords1, const FragCoords *coords2, double *coeff);
@@ -34,4 +35,7 @@ ComputeQuartHornFrag(const FragCoords *coords1, const FragCoords *coords2, doubl
 double
 QuarticHornFrag(const FragCoords *coords1, const FragCoords *coords2, double *coeff);
 
+void
+FragDistPu(CoordsArray *cdsA, int fraglen, int center_ca, int pu);
+
 #endif
