@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ average(const double *data, const int dim)
 
 
 double
-variance(const double *data, const int dim, const double mean)
+Variance(const double *data, const int dim, const double mean)
 {
     double          v = 0.0, tmpv;
     int             i = dim;
@@ -92,7 +92,7 @@ variance(const double *data, const int dim, const double mean)
    i.e. we are interested in the variance of the estimate, not the
    distribution.*/
 double
-varvariance(const double *data, const int ddim, const int sdim)
+VarVariance(const double *data, const int ddim, const int sdim)
 {
     double          v = 0.0, tmpv;
     int             i = ddim;
@@ -107,6 +107,7 @@ varvariance(const double *data, const int ddim, const int sdim)
     /* return(2.0 * v / (ddim * sdim * sdim)); */
 }
 
+
 int
 moments(const double *data, const int dim, /* data array and length */
         double *ave, double *median,       /* mean and median */
