@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -489,7 +489,7 @@ VerifyRotMat(double **rotmat, double tol)
 
         if (fabs(sum1 - 1.0) > tol || fabs(sum2 - 1.0) > tol)
         {
-            MatDestroy(testmat);
+            MatDestroy(&testmat);
             printf(" ERROR: rotation matrix not normalized\n");
             printf("        row %d sum = %f, column %d sum = %f\n", i, sum1, j, sum2);
             fflush(NULL);
@@ -519,7 +519,7 @@ VerifyRotMat(double **rotmat, double tol)
                 printf(" Matrix multiplied by self-transpose should be identity:\n");
                 Mat3Print(testmat);
                 fflush(NULL);
-                MatDestroy(testmat);
+                MatDestroy(&testmat);
                 return(0);
             }
 
@@ -531,13 +531,13 @@ VerifyRotMat(double **rotmat, double tol)
                 printf(" Matrix multiplied by self-transpose should be identity:\n");
                 Mat3Print(testmat);
                 fflush(NULL);
-                MatDestroy(testmat);
+                MatDestroy(&testmat);
                 return(0);
             }
         }
     }
 
-    MatDestroy(testmat);
+    MatDestroy(&testmat);
     return(1);
 }
 
@@ -575,8 +575,8 @@ double
 
     printf(" det = %e\n", Mat3Det((const double **)rotmat));
 
-    MatDestroy(u);
-    MatDestroy(vt);
+    MatDestroy(&u);
+    MatDestroy(&vt);
     free(s);
 
     return(rotmat);
@@ -604,8 +604,8 @@ ClosestRotMatIp(double **inmat)
 
     Mat3Print(inmat);
 
-    MatDestroy(u);
-    MatDestroy(vt);
+    MatDestroy(&u);
+    MatDestroy(&vt);
     free(s);
 }
 
