@@ -1310,6 +1310,10 @@ InvSymEigenOp(double **invmat, const double **mat, long int n,
 
 /* Entire decomposition, evals and evecs */
 /* eigensym evals are small to large */
+/* Symmetric matrix decomposition, using LAPACK dsyev
+   Obviously it doesn't matter if you pass the matrix or its transpose;
+   however, LAPACK matrix ordering is FORTRAN-style, not C, so the
+   eigenvectors need to be transposed. */
 void
 eigensym(const double **mat, double *evals, double **evecs, long int n)
 {
