@@ -1,9 +1,9 @@
 /***********************************************************************
 *  -/_|:|_|_\- 
 *                                                                      *
-*  dssplite v.0.7                                                      *
+*  dssplite v.0.8                                                      *
 *                                                                      *
-*  Copyright (C) 2003-2007  Douglas L. Theobald                        *
+*  Copyright (C) 2003-2008  Douglas L. Theobald                        *
 *                                                                      *
 *  dssplite is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published   *
@@ -200,7 +200,7 @@ PrintSummary(dssp);
     summary = (char *) malloc(len * sizeof(char));
     strncpy(summary, dssp->summary, len);
 
-    DSSPfree(dssp);
+    DSSPfree(&dssp);
 
     return(summary);
 }
@@ -828,7 +828,7 @@ Dihedralangle(double **Ad, int An,
 DSSP
 *DSSPinit(void)
 {
-    DSSP        *dssp;
+    DSSP        *dssp = NULL;
 
     dssp = (DSSP *) malloc(sizeof(DSSP));
 
@@ -971,8 +971,10 @@ DSSPalloc(DSSP *dssp, int len)
 
 
 void
-DSSPfree(DSSP *dssp)
+DSSPfree(DSSP **dssp_ptr)
 {
+    DSSP *dssp = *dssp_ptr;
+
     free(dssp->C[0]);
     free(dssp->C);
     free(dssp->O[0]);
@@ -1005,7 +1007,7 @@ DSSPfree(DSSP *dssp)
     free(dssp->disulph);
     free(dssp->summary);
     free(dssp);
-    dssp = NULL;
+    *dssp_ptr = NULL;
 }
 
 
@@ -1039,12 +1041,12 @@ PrintSummary(DSSP *dssp)
     int             i;
     int             len = dssp->len;
 
-    printf(  "\n\n atom[num] res SS [H-bnd] - B1 B2 S T3 T4 T5 ");
-    printf(    "\n --------- --- -- -------   -- -- - -- -- -- ");
+    printf(  "\n\n atom[num] res SS [H-bnd] -                 - B1 B2  S T3 T4 T5 \n");
+    printf(    " --------- --- -- -------                     -- -- -- -- -- -- \n");
 
     for (i = 0; i < len; ++i)
     {
-        printf(" atom[%3d] %3s %c  %3d %3d - %3d %3d %3d %3d - %2d %2d %1d %2d %2d %2d \n",
+        printf(" atom[%3d] %3s %c  %3d %3d - %3d %3d %3d %3d - %2d %2d %2d %2d %2d %2d \n",
                 i+1, dssp->residue[i], dssp->summary[i],
                 dssp->bondatm[i][0]+1, dssp->bondatm[i][1]+1,
                 dssp->bridgeP[i][0], dssp->bridgeP[i][1], dssp->bridgeA[i][0], dssp->bridgeA[i][1],
