@@ -616,7 +616,7 @@ MSA
                         if(msa->seq[i][k] != msa->seq[j][k])
                             break;
                     }
-        
+
                     if(k == msa->seqlen)
                         msa->flag[j] = 1;
                 }
@@ -656,6 +656,7 @@ MSA
 
 
 /* delete all gapped columns above a specified fraction cutoff, within a range of columns and sequences */
+/* frac specifies the maximum fraction of gaps allowed per column */
 MSA
 *MSAdelallgaps(MSA *msa, int begcol, int endcol, int logap, int upgap, double frac)
 {
@@ -697,10 +698,14 @@ MSA
         //if (j == upgap)
         //printf("\n%d %f", i+1, cnt/msa->seqnum);
         //if (cnt/msa->seqnum <= frac)
+        /* if the fraction of gaps for this column is less than the specified cutoff, keep it */
         if (cnt / (upgap - logap + 1.0) <= frac)
         {
             for (k = 0; k < msa->seqnum; ++k)
+            {
                 tmpmsa->seq[k][m] = msa->seq[k][i];
+                //printf("\n%d %c %d %c", m, tmpmsa->seq[k][m], i, msa->seq[k][i]);
+            }
             ++m;
         }
     }
@@ -712,8 +717,11 @@ MSA
 //printf("\n### endcol:%d m:%d", endcol, m);
 
     for (k = 0; k < msa->seqnum; ++k)
+    {
+        //printf("\n%d %c", m, tmpmsa->seq[k][m]);
         tmpmsa->seq[k][m] = '\0';
-    tmpmsa->seqlen = m - 1;
+    }
+    tmpmsa->seqlen = m;
 
     MSAcpy(msa, tmpmsa);
     MSAfree(&tmpmsa);
@@ -851,7 +859,7 @@ MSA
    Indexes of the matching sequences are in seq1 and seq2.
    The starting and ending indexes of the matches are in
    begin1, end1, begin2, end2.
-   
+
    NOTE:  end1 and end2 are inclusive. */
 int
 MSAseqeq(MSA *msa1,
@@ -1844,7 +1852,7 @@ nexus_dist(MSA *msa, char *filestr, int filesize, int format)
             }
             ++j;
         }
-        
+
         msa->name[i][j+1] = '\0';
 
         if (strlen(msa->name[i]) < 1) 
@@ -1928,7 +1936,7 @@ nexus_dist(MSA *msa, char *filestr, int filesize, int format)
    whitespace between key and value is ignored. 
    These are used in 'readnex()' for NEXUS format stuff.
    Note the return types. 
-   
+
    The 'endptr' argument can be NULL, but if the value
    is not found endptr = &string, so you can check. 
    getvalc() only returns isgraph() characters 
@@ -2266,7 +2274,7 @@ phylip_dist(MSA *msa, char *filestr, int filesize, int format)
         /* read the name, id, etc. first */
         for (j = 0; j < 10 && k < filesize; ++j, ++k)
             msa->name[i][j] = filestr[k];
-        
+
         j=9; /* get rid of trailing spaces */
         while(j >= 0 && isspace(msa->name[i][j]))
             --j;
