@@ -90,7 +90,7 @@ MSAA
 *MSAAalloc(int arraynum);
 
 void
-MSAfree(MSA **msa);
+MSAfree(MSA **msa_ptr);
 
 void
 MSAAfree(MSAA **msas_ptr);
