@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -295,7 +295,9 @@ ReadPDBCoords(char *pdbfile_name, PDBCoordsArray *pdbA, int cds_i, int modelnum)
         {
             while (fgets(buff, sizeof(buff), pdbfile) != NULL)
             {
-                if (strncmp(buff, "MODEL", 5) == 0)
+                if (strncmp(buff, "MODEL", 5) == 0 || 
+                    strncmp(buff, "REMARK FRAME:", 13) == 0 ||
+                    strncmp(buff, "REMARK  OPLS-AA", 15) == 0) // DLTPU
                 {
                     cp->model = i;
                     break;
@@ -311,7 +313,7 @@ ReadPDBCoords(char *pdbfile_name, PDBCoordsArray *pdbA, int cds_i, int modelnum)
         {
             if (j >= cp->vlen ||
                 strncmp(buff, "ENDMDL", 6) == 0 ||
-                strncmp(buff, "END   ", 6) == 0)
+                strncmp(buff, "END", 3) == 0)
             {
                 break;
             }
@@ -331,7 +333,7 @@ ReadPDBCoords(char *pdbfile_name, PDBCoordsArray *pdbA, int cds_i, int modelnum)
         }
     }
 
-    //PrintPDBCoords(stdout, pdbA->cds[0]);
+    //PrintPDBCoords(stdout, pdbA->coords[0]);
     fclose(pdbfile);
     return(EXIT_SUCCESS);
 }
@@ -435,8 +437,259 @@ ReadPDBCoords(char *pdbfile_name, PDBCoordsArray *pdbA, int cds_i, int modelnum)
 /* } */
 
 
+static int
+GetSlxnLen(PDBCoordsArray *pdbA, const int crds, Algorithm *algo, char *chains,
+           int *lower, int *upper, const int range_num, int *selection_index, int rev)
+{
+    int            i, vlen;
+
+    vlen = 0;
+	for (i = 0; i < pdbA->coords[crds]->vlen; ++i)
+	{
+/*         printf("\n%s, %s, %c, %c, %d", */
+/*                pdbA->coords[crds]->record[i], */
+/*                pdbA->coords[crds]->name[i], */
+/*                pdbA->coords[crds]->altLoc[i], */
+/*                pdbA->coords[crds]->chainID[i], */
+/*                pdbA->coords[crds]->resSeq[i]); */
+/*         fflush(NULL); */
+		if (   (   strncmp(pdbA->coords[crds]->record[i], "ATOM  ", 6) == 0
+				|| strncmp(pdbA->coords[crds]->record[i], "HETATM", 6) == 0)
+			&& atom_selxn(pdbA->coords[crds]->name[i], algo->atoms, algo->atomslxn) == 1
+			&& (pdbA->coords[crds]->altLoc[i] == ' ' || pdbA->coords[crds]->altLoc[i] == 'A')
+			&& range_selxn(pdbA->coords[crds]->chainID[i], chains, pdbA->coords[crds]->resSeq[i], lower, upper, range_num) == rev) /* DLT debug FIX THIS!!!! */
+		{
+		    if (selection_index != NULL)
+    			selection_index[vlen] = i;
+			++vlen;
+		}
+	}
+
+    return(vlen);
+}
+
+
 void
 GetCoordsSelection(CoordsArray *baseA, PDBCoordsArray *pdbA)
+{
+    int             vlen, last_len;
+    int            *selection_index = NULL; /* array of ints corresponding to selected atoms */
+    int             i, j;
+    char           *chains = NULL; /* array of chars holding chain IDs of ranges in the selection criteria */
+    int            *upper = NULL, *lower = NULL; /* an array of ints holding the upper and lower range bounds */
+    int             selection_len;
+    char          **endptr = NULL;
+    int             range_num = 1; /* number of residue ranges specified in selection */
+    char          **selections = NULL; /* an array of range_num strings to hold each range selection */
+    char            delims[] = ":";
+
+    if (baseA->algo->selection != NULL)
+    {
+        selection_len = strlen(baseA->algo->selection);
+
+        for(i = 0; i < selection_len; ++i)
+        {
+            if (baseA->algo->selection[i] == ':')
+                ++range_num;
+        }
+
+        selections = (char **) calloc(range_num, sizeof(char *));
+        lower      = (int *)   calloc(range_num, sizeof(int));
+        upper      = (int *)   calloc(range_num, sizeof(int));
+        chains     = (char *)  calloc(range_num, sizeof(char));
+        if (selections == NULL || lower == NULL || upper == NULL || chains == NULL)
+        {
+            perror("\n  ERROR");
+            fprintf(stderr, "\n  ERROR1000: could not allocate memory for selections in GetCoordsSelection(). \n");
+            PrintTheseusTag();
+            exit(EXIT_FAILURE);
+        }
+        
+        for (i = 0; i < range_num; ++i)
+        {
+            selections[i] = (char *) calloc(128, sizeof(char));
+            if (selections[i] == NULL)
+            {
+                perror("\n  ERROR");
+                fprintf(stderr, "\n  ERROR1001: could not allocate memory for selections[] in GetCoordsSelection(). \n");
+                PrintTheseusTag();
+                exit(EXIT_FAILURE);
+            }   
+        }
+
+        /* if the user specified XPLOR/CNS-style atoms, translate them to standard PDB format (DNA primes should be "*", not "'") */
+        for (i = 0; i < strlen(baseA->algo->selection); ++i)
+            if (baseA->algo->selection[i] == '\'')
+                baseA->algo->selection[i] = '*';
+
+        /* copy each range selection string into the 'selections[]' array */
+        mystrncpy(selections[0], strtok(baseA->algo->selection, delims), 127); 
+        for (i = 1; i < range_num; ++i)
+            mystrncpy(selections[i], strtok(NULL, delims), 127);
+
+/*         for (i = 0; i < range_num; ++i) */
+/*             printf"\n selections[%d] = %s", i, selections[i]); */
+
+        for (j = 0; j < range_num; ++j)
+        {
+            /* parse residue number range */
+            selection_len = strlen(selections[j]);
+
+            i = 0;
+            while(isspace(selections[j][i]) && i < selection_len)
+                ++i;
+
+            if (isalpha(selections[j][i]))
+            {
+                chains[j] = toupper(selections[j][i]);
+                ++i;
+            }
+            else
+                chains[j] = 0;
+
+            if (isalpha(selections[j][i]))
+            {
+                fprintf(stderr, "\n  ERROR1002: incorrect format for chainID selection (too many characters). \n");
+                Usage(0);
+                exit(EXIT_FAILURE);
+            }
+
+            if (isdigit(selections[j][i]))
+            {
+                lower[j] = (int) strtol(&selections[j][i], endptr, 10);
+        
+                while(selections[j][i] != '-' && i < selection_len)
+                    ++i;
+    
+                ++i;
+                while(isspace(selections[j][i]) && i < selection_len)
+                    ++i;
+    
+                upper[j] = (int) strtol(&selections[j][i], endptr, 10);
+            }
+            else
+            {
+                lower[j] = 0;
+                upper[j] = pdbA->vlen - 1;
+            }
+        }
+    }
+    else
+    {
+        range_num = 1;
+        selections = (char **) calloc(1, sizeof(char *));
+        lower      = (int *)   calloc(1, sizeof(int));
+        upper      = (int *)   calloc(1, sizeof(int));
+        chains     = (char *)  calloc(1, sizeof(char));
+        selections[0] = (char *) calloc(128, sizeof(char));
+        if (selections == NULL || lower == NULL || upper == NULL || chains == NULL || selections[0] == NULL)
+        {
+            perror("\n  ERROR");
+            fprintf(stderr, "\n  ERROR1003: could not allocate memory for selections in GetCoordsSelection(). \n");
+            PrintTheseusTag();
+            exit(EXIT_FAILURE);
+        }
+
+        lower[0] = -INT_MAX;
+        upper[0] = INT_MAX;
+    }
+
+/*     if (upper[0] - 4 < lower[0]) */
+/*     { */
+/*         fprintf(stderr, "\n  ERROR876: upper residue bound must be at least 3 greater than the lower bound. %d %d \n", */
+/*                          upper[0], lower[0]); */
+/*         PrintTheseusTag(); */
+/*         exit(EXIT_FAILURE); */
+/*     } */
+
+    selection_index = (int *) calloc (pdbA->vlen, sizeof(int));
+    if (selection_index == NULL)
+    {
+        perror("\n  ERROR");
+        fprintf(stderr, "\n  ERROR1004: could not allocate memory for selection_index in GetCoordsSelection(). \n");
+        PrintTheseusTag();
+        exit(EXIT_FAILURE);
+    }
+
+    /* First count the number of selected atoms for each structure, and make sure they are all the same */
+    last_len = vlen = 0;
+    for (i = 0; i < pdbA->cnum; ++i)
+    {
+/*         printf("\n%s, %s, %c, %c, %d", */
+/*                pdbA->coords[i]->record[i], */
+/*                pdbA->coords[i]->name[i], */
+/*                pdbA->coords[i]->altLoc[i], */
+/*                pdbA->coords[i]->chainID[i], */
+/*                pdbA->coords[i]->resSeq[i]); */
+/*         fflush(NULL); */
+        last_len = vlen;
+		if (baseA->algo->revsel == 0)
+			vlen = GetSlxnLen(pdbA, i, baseA->algo, chains, lower, upper, range_num, NULL, 1);
+		else
+			vlen = GetSlxnLen(pdbA, i, baseA->algo, chains, lower, upper, range_num, NULL, 0);
+	
+	    if (i > 0 && vlen != last_len)
+		{
+			fprintf(stderr,
+					"\n  ERROR1: length of selection is (%d) different from preceding coords (%d) in GetCoordsSelection() \n", vlen, i+1);
+			PrintTheseusTag();
+			exit(EXIT_FAILURE);
+		}
+
+		if (vlen < 3)
+		{
+			fprintf(stderr,
+					"\n  ERROR3: 'vlen' is too small (%d) in GetCoordsSelection(); not enough atoms selected in coords %d. \n", vlen, i+1);
+			PrintTheseusTag();
+			exit(EXIT_FAILURE);
+		}
+    }
+
+    /* Now allocate based on this length */
+    /* baseA = CoordsArrayInit(); */ /* NB!!: we don't need to initialize, since the pointer was passed above */
+    CoordsArrayAlloc(baseA, pdbA->cnum, vlen);
+    CoordsArraySetup(baseA);
+
+    /* Now get the selections again and read them in this time */
+    for (j = 0; j < pdbA->cnum; ++j)
+    {
+		if (baseA->algo->revsel == 0)
+			vlen = GetSlxnLen(pdbA, j, baseA->algo, chains, lower, upper, range_num, selection_index, 1);
+		else
+			vlen = GetSlxnLen(pdbA, j, baseA->algo, chains, lower, upper, range_num, selection_index, 0);
+
+        baseA->coords[j]->model = pdbA->coords[j]->model;
+        strncpy(baseA->coords[j]->filename, pdbA->coords[j]->filename, FILENAME_MAX - 1);
+
+        for (i = 0; i < baseA->vlen; ++i)
+        {
+            strncpy(baseA->coords[j]->resName[i], pdbA->coords[j]->resName[selection_index[i]], 3);
+            baseA->coords[j]->chainID[i] = pdbA->coords[j]->chainID[selection_index[i]];
+            baseA->coords[j]->resSeq[i] = pdbA->coords[j]->resSeq[selection_index[i]];
+            baseA->coords[j]->x[i]      = pdbA->coords[j]->x[selection_index[i]];
+            baseA->coords[j]->y[i]      = pdbA->coords[j]->y[selection_index[i]];
+            baseA->coords[j]->z[i]      = pdbA->coords[j]->z[selection_index[i]];
+            baseA->coords[j]->o[i]      = 1.0;/* pdbA->coords[j]->occupancy[selection_index[i]]; */ /* DLT debug - shouldn't need to do this, I think prob is in SuperPose() */
+            baseA->coords[j]->b[i]      = pdbA->coords[j]->tempFactor[selection_index[i]];
+        }
+    }
+    
+/* PrintCoords(baseA->coords[0]); */
+/* PrintCoords(baseA->coords[1]); */
+
+    for (i = 0; i < range_num; ++i)
+        free(selections[i]);
+
+    free(selections);
+    free(selection_index);
+    free(upper);
+    free(lower);
+    free(chains);
+}
+
+
+void
+GetCoordsSelection_old(CoordsArray *baseA, PDBCoordsArray *pdbA)
 {
     int             vlen;
     int            *selection_index = NULL; /* array of ints corresponding to selected atoms */
@@ -669,6 +922,8 @@ GetCoordsSelection(CoordsArray *baseA, PDBCoordsArray *pdbA)
 }
 
 
+
+
 PDBCoordsArray
 *GetPDBCoords(char **argv_array, int narguments, int fmodel)
 {
@@ -725,12 +980,14 @@ PDBCoordsArray
         last_vlen = 0;
         while (fgets(buff, (int) sizeof(buff), pdbfile) != NULL)
         {
-            if (strncmp(buff, "MODEL", 5) == 0)
+            if (strncmp(buff, "MODEL", 5) == 0 || 
+                strncmp(buff, "REMARK FRAME:", 13) == 0 ||
+                strncmp(buff, "REMARK  OPLS-AA", 15) == 0) // DLTPU
             {
                 vlen = 0;
                 while (fgets(buff, sizeof(buff), pdbfile))
                 {
-                    if (strncmp(buff, "ENDMDL", 6) == 0)
+                    if ((strncmp(buff, "END", 3) == 0) || (strncmp(buff, "ENDMDL", 6) == 0)) // DLTPU
                         break;
                     else if (strncmp(buff, "ATOM  ", 6) == 0 || strncmp(buff, "HETATM", 6) == 0)
                         ++vlen;
@@ -742,7 +999,7 @@ PDBCoordsArray
                 else if ((models > 0) && (vlen != last_vlen))
                 {
                     fprintf(stderr,
-                            "\n  ERROR1005: coords #%d in \"%s\" has a different atom number (%d) than the preceding coords (%d). \n",
+                            "\n  WARNING1005: coords #%d in \"%s\" has a different atom number (%d) than the preceding coords (%d). \n",
                             models + 1, pdbfile_name, vlen, last_vlen);
 /*                     PrintTheseusTag(); */
 /*                     exit(EXIT_FAILURE); */
@@ -750,7 +1007,7 @@ PDBCoordsArray
                 else if (vlen < 3)
                 {
                     fprintf(stderr,
-                            "\n  ERROR1006: coords #%d in \"%s\" has no selected atoms. \n",
+                            "\n  WARNING1006: coords #%d in \"%s\" has no selected atoms. \n",
                             models + 1, pdbfile_name);
                     PrintTheseusTag();
                     exit(EXIT_FAILURE);
@@ -783,7 +1040,7 @@ PDBCoordsArray
 
             while (fgets(buff, sizeof(buff), pdbfile) != NULL)
             {
-                if (strncmp(buff, "END   ", 6) == 0 || (strncmp(buff, "ENDMDL", 6) == 0))
+                if (strncmp(buff, "END", 3) == 0 || (strncmp(buff, "ENDMDL", 6) == 0))
                     break;
                 else if (strncmp(buff, "ATOM  ", 6) == 0 || strncmp(buff, "HETATM", 6) == 0)
                     ++vlen;
@@ -801,7 +1058,7 @@ PDBCoordsArray
         }
 
         len_array[i] = vlen;
-        //printf("\n vlen = %d", vlen);
+        //printf("\n vlen[%d] = %d", i, vlen);
     }
 
     pdbA = PDBCoordsArrayInit();
