@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -670,7 +670,7 @@ WriteVariance(CoordsArray *cdsA, char *outfile_name)
         exit(EXIT_FAILURE);
     }
 
-    fprintf(varfile, "#ATOM   resName resSeq     variance      std dev         RMSD\n");
+    fprintf(varfile, "#ATOM   resName resSeq     variance      std_dev         RMSD\n");
 
     if (cdsA->algo->varweight == 1 || cdsA->algo->leastsquares == 1)
     {
