@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2008 Douglas L. Theobald
+    Copyright (C) 2004-2009 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -351,10 +351,32 @@ NMRCheckPDBCoordsArray(PDBCoordsArray *pdbA)
         if (pdbA->coords[i]->vlen != vlen)
         {
             fprintf(stderr,
-                    "\n  ERROR20: PDB coordinates %d and %d are of unequal length [%d vs %d]. \n",
+                    "\n  WARNING20: PDB coordinates %d and %d are of unequal length [%d vs %d]. \n\n",
                     0, i, vlen, pdbA->coords[i]->vlen);
-            PrintTheseusTag();
-            exit(EXIT_FAILURE);
+            //PrintTheseusTag();
+            //exit(EXIT_FAILURE);
+        }
+    }
+
+    return(vlen);
+}
+
+
+int
+NMRCheckCoordsArray(CoordsArray *pdbA)
+{
+    int             i;
+    int             vlen = pdbA->coords[0]->vlen;
+
+    for(i = 1; i < pdbA->cnum; ++i)
+    {
+        if (pdbA->coords[i]->vlen != vlen)
+        {
+            fprintf(stderr,
+                    "\n  WARNING20: PDB coordinates %d and %d are of unequal length [%d vs %d]. \n\n",
+                    0, i, vlen, pdbA->coords[i]->vlen);
+            //PrintTheseusTag();
+            //exit(EXIT_FAILURE);
         }
     }
 
@@ -1196,6 +1218,126 @@ AveCoordsNovec(CoordsArray *cdsA)
 }
 
 
+/* Calculate the ML estimate of a hierarchical mean, where the atoms
+   are normally distributed with hyper-mean zero */
+/* See also below, which is probably more valid (only the weighted mean has zero centroid) */
+/* 2009-06-11 */
+double
+HierAveCoords2(CoordsArray *cdsA)
+{
+    int             i, j;
+    double         *avex = cdsA->avecoords->x,
+                   *avey = cdsA->avecoords->y,
+                   *avez = cdsA->avecoords->z,
+                   *aveo = cdsA->avecoords->o,
+                   *aveb = cdsA->avecoords->b;
+    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
+    const Coords  **coords = (const Coords **) cdsA->coords;
+    Coords         *coordsj;
+    double          invcnum, psi;
+
+
+    psi = 0.0;
+    for (i = 0; i < vlen; ++i)
+        psi += (avex[i]*avex[i] + avey[i]*avey[i] + avez[i]*avez[i]);
+    psi /= (3.0 * vlen);
+
+    memset(avex, 0, vlen * sizeof(double));
+    memset(avey, 0, vlen * sizeof(double));
+    memset(avez, 0, vlen * sizeof(double));
+    memset(aveb, 0, vlen * sizeof(double));
+
+    for (j = 0; j < cnum; ++j)
+    {
+        coordsj = (Coords *) coords[j];
+
+		for (i = 0; i < vlen; ++i)
+		{
+			avex[i] += coordsj->x[i];
+			avey[i] += coordsj->y[i];
+			avez[i] += coordsj->z[i];
+			aveb[i] += coordsj->b[i];
+		}
+    }
+
+    for (i = 0; i < vlen; ++i)
+    {
+        invcnum = 1.0 / ((double) cnum + cdsA->var[i] / psi);
+        //printf("\ninvcnum = %e  %e", invcnum, 1.0/cnum);
+    
+		avex[i] *= invcnum;
+		avey[i] *= invcnum;
+		avez[i] *= invcnum;
+		aveo[i] = 1.0;
+		aveb[i] *= invcnum;
+    }
+    
+    return(psi);
+}
+
+
+/* Calculate the ML estimate of a hierarchical mean, where the variance-weighted atoms 
+   are normally distributed with hyper-mean zero */
+/* 2009-06-11 */
+double
+HierAveCoords(CoordsArray *cdsA)
+{
+    int             i, j;
+    double         *avex = cdsA->avecoords->x,
+                   *avey = cdsA->avecoords->y,
+                   *avez = cdsA->avecoords->z,
+                   *aveo = cdsA->avecoords->o,
+                   *aveb = cdsA->avecoords->b;
+    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
+    const Coords  **coords = (const Coords **) cdsA->coords;
+    Coords         *coordsj;
+    double          invcnum, psi, norm;
+
+
+    psi = norm = 0.0;
+    for (i = 0; i < vlen; ++i)
+    {
+        norm += 1.0 / cdsA->var[i];
+        psi += (avex[i]*avex[i]/cdsA->var[i] + 
+                avey[i]*avey[i]/cdsA->var[i] + 
+                avez[i]*avez[i]/cdsA->var[i]);
+    }
+    psi /= (3.0 * norm);
+
+    memset(avex, 0, vlen * sizeof(double));
+    memset(avey, 0, vlen * sizeof(double));
+    memset(avez, 0, vlen * sizeof(double));
+    memset(aveb, 0, vlen * sizeof(double));
+
+    for (j = 0; j < cnum; ++j)
+    {
+        coordsj = (Coords *) coords[j];
+
+		for (i = 0; i < vlen; ++i)
+		{
+			avex[i] += coordsj->x[i];
+			avey[i] += coordsj->y[i];
+			avez[i] += coordsj->z[i];
+			aveb[i] += coordsj->b[i];
+		}
+    }
+
+    for (i = 0; i < vlen; ++i)
+    {
+        invcnum = 1.0 / ((double) cnum + 1.0 / psi);
+        //printf("\ninvcnum = %e  %e", invcnum, 1.0/cnum);
+    
+		avex[i] *= invcnum;
+		avey[i] *= invcnum;
+		avez[i] *= invcnum;
+		aveo[i] = 1.0;
+		aveb[i] *= invcnum;
+    }
+    
+    return(psi);
+}
+
+
 void
 AveCoords(CoordsArray *cdsA)
 {
