@@ -16,7 +16,7 @@
  *                  dtheobald@gmail.com
  *                  dtheobald@brandeis.edu
  *
- *  Copyright:      Copyright (c) 2004-2008 Douglas L. Theobald
+ *  Copyright:      Copyright (c) 2004-2009 Douglas L. Theobald
  *
  *  THESEUS is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as 
@@ -231,6 +231,21 @@ main(int argc, char *argv[])
         /* PrintPDBCoords(stdout, pdbA->coords[0]); */
     }
 
+    if (algo->fasta == 1)
+    {
+        if (pdbA->cnum < 1)
+        {
+            printf("\n  -> Found no PDB coords. Could not determine a sequence. <- \n");
+            Usage(0);
+            exit(EXIT_FAILURE);
+        }
+
+        printf("    Writing FASTA format .fst files (%d) ... \n", pdbA->cnum);
+        pdb2fst(pdbA);
+        PrintTheseusTag();
+        exit(EXIT_SUCCESS);
+    }
+
     if (pdbA->cnum < 2)
     {
         printf("\n  -> Found less than two PDB coords. Could not do superposition. <- \n");
@@ -249,13 +264,6 @@ main(int argc, char *argv[])
         PrintTheseusTag();
         exit(EXIT_SUCCESS);
     }
-    else if (algo->fasta == 1)
-    {
-        printf("    Writing FASTA format .fst files (%d) ... \n", pdbA->cnum);
-        pdb2fst(pdbA);
-        PrintTheseusTag();
-        exit(EXIT_SUCCESS);
-    }
     else if (algo->alignment == 1)
     {
         printf("    Reading multiple sequence alignment ... \n");
@@ -297,9 +305,13 @@ main(int argc, char *argv[])
     }
 
     if (algo->jackknife > 0)
+    {
         SuperJack(baseA);
+    }
     else if (algo->FragDist > 0)
-        FragDist(baseA, algo->FragDist, algo->FragDist);
+    {
+        FragDistPu(baseA, algo->FragDist, 2, algo->pu);
+    }
     else if (algo->info == 1)
     {
         printf("    Calculating superposition statistics ... \n");
@@ -448,6 +460,32 @@ main(int argc, char *argv[])
             PrintCovMatGnuPlot((const double **) baseA->avecoords->innerprod, baseA->vlen, mystrcat(algo->rootname, "_mean_ip.mat"));
         }
     }
+    
+/*     FILE *fp = fopen("pdbdists.txt" ,"w"); */
+/*     double xx,yy,zz; */
+/*     double vv, vsum; */
+/*      */
+/*     vsum = 0.0; */
+/*     for (j = 0; j < baseA->vlen; ++j) */
+/*         vsum += 1.0/baseA->var[j];; */
+/*      */
+/*     for (i = 0; i < baseA->cnum; ++i) */
+/*     {i=0; */
+/*         for (j = 0; j < baseA->vlen; ++j) */
+/*         { */
+/*             xx = baseA->coords[i]->x[j]; */
+/*             yy = baseA->coords[i]->y[j]; */
+/*             zz = baseA->coords[i]->z[j]; */
+/*             vv = vsum / baseA->var[j]; */
+/*             vv=1.0; */
+/*             //fprintf(fp, "%8.3e\n", xx*xx+yy*yy+zz*zz); */
+/*             fprintf(fp, "%8.3e\n", xx/vv); */
+/*             fprintf(fp, "%8.3e\n", yy/vv); */
+/*             fprintf(fp, "%8.3e\n", zz/vv); */
+/*         } */
+/*         break; */
+/*     } */
+/*      fprintf(fp, "\n\n"); */
 
     PrintTheseusTag();
 
@@ -594,12 +632,16 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *bseA)
             case 'C':
                 algo->cormat = 0;
                 break;
-            case 'd':
-                algo->scale = (double) strtod(optarg, NULL); /* specify a type of scaling to do */
-                //algo->dimweight = 1;
-                break;
-            case 'D':
+//            case 'd':
+//                 algo->scale = (double) strtod(optarg, NULL); /* specify a type of scaling to do */                 //algo->dimweight = 1; */
+//                 break;
+            case 'd': /* FragDist benchmarking, LAPACK rotation calc */
                 algo->FragDist = (int) strtol(optarg, NULL, 10);
+                algo->pu = 0;
+                break;
+            case 'D': /* FragDist benchmarking, Pu/Theobald QCP rotation calc */
+                algo->FragDist = (int) strtol(optarg, NULL, 10);
+                algo->pu = 1;
                 break;
             case 'e':
                 algo->embedave = (int) strtol(optarg, NULL, 10);
@@ -621,7 +663,9 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *bseA)
                     algo->hierarch = 3;
                 break;
             case 'G':
-                algo->fullpca = 1;
+                //algo->fullpca = 1;
+                // find rotations with QCP algorithm
+                algo->pu = 1;
                 break;
             case 'h':
                 Usage(0);
@@ -905,7 +949,7 @@ PrintSuperposStats(CoordsArray *cdsA)
 /*            normtest, chisqr_sdf(normtest, 2.0, 0)); */
 
     printf("  FP error in transformed coordinates:             %3.2e\n", stats->fperr);
-    printf("  Minimum RMSD error per atom:                     %3.2e\n", stats->minvar);
+    printf("  Minimum RMS error per atom:                      %3.2e\n", stats->minvar);
     printf("  Data pts = %d,  Free params = %d,  D/P = %-5.1f\n",
            (int) stats->ndata, (int) stats->nparams, (stats->ndata / stats->nparams));
 
